@@ -1,8 +1,44 @@
 #!/usr/bin/perl
+## --------------------------------------------------------------------------
+##   
+##   Copyright 1996-2009 The NASM Authors - All Rights Reserved
+##   See the file AUTHORS included with the NASM distribution for
+##   the specific copyright holders.
+##
+##   Redistribution and use in source and binary forms, with or without
+##   modification, are permitted provided that the following
+##   conditions are met:
+##
+##   * Redistributions of source code must retain the above copyright
+##     notice, this list of conditions and the following disclaimer.
+##   * Redistributions in binary form must reproduce the above
+##     copyright notice, this list of conditions and the following
+##     disclaimer in the documentation and/or other materials provided
+##     with the distribution.
+##     
+##     THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
+##     CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
+##     INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
+##     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+##     DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
+##     CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+##     SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
+##     NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
+##     LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+##     HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+##     CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
+##     OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
+##     EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+##
+## --------------------------------------------------------------------------
+
 #
 # Format the documentation as PostScript
 #
 
+use Env;
+use lib $srcdir;
+
 require 'psfonts.ph';		# The fonts we want to use
 require 'pswidth.ph';		# PostScript string width
 
@@ -78,7 +114,7 @@ while ( $arg = shift(@ARGV) ) {
 
 #
 # Document formatting parameters
-# 
+#
 $paraskip = 6;			# Space between paragraphs
 $chapstart = 30;		# Space before a chapter heading
 $chapskip = 24;			# Space after a chapter heading
@@ -204,7 +240,7 @@ sub int2base($$) {
 	$i = int($i/$b);
     }
     return $n.$s;
-}    
+}
 
 #
 # Convert a string to a rendering array
@@ -213,8 +249,9 @@ sub string2array($)
 {
     my($s) = @_;
     my(@a) = ();
-    
-    $s =~ s/ \- / $charcode{'endash'} /g;	# Replace " - " with en dash
+
+    $s =~ s/\B\-\-\B/$charcode{'emdash'}/g;
+    $s =~ s/\B\-\B/ $charcode{'endash'} /g;
 
     while ( $s =~ /^(\s+|\S+)(.*)$/ ) {
 	push(@a, [0,$1]);
@@ -280,7 +317,7 @@ sub ps_flow_lines($$$@) {
     foreach $e ( @data ) {
 	if ( $$e[0] < 0 ) {
 	    # Type is metadata.  Zero width.
-	    if ( $$e[0] == -6 ) { 
+	    if ( $$e[0] == -6 ) {
 		$pastmarker = 1;
 	    }
 	    if ( $$e[0] == -1 || $$e[0] == -6 ) {
@@ -299,7 +336,7 @@ sub ps_flow_lines($$$@) {
 	    my $esw = ps_width($sp, $fontset->{fonts}->[$$e[0]][1],
 			       \@NASMEncoding) *
 		($fontset->{fonts}->[$$e[0]][0]/1000);
-	    
+
 	    if ( ($w+$ew) - $ps_space_squeeze*($sw+$esw) > $wid ) {
 		# Begin new line
 		# Search backwards for previous space chunk
@@ -398,7 +435,7 @@ sub ps_merge_chunks(@) {
     my(@ci) = @_;
     my($c, $lc);
     my(@co, $eco);
-    
+
     undef $lc;
     @co = ();
     $eco = -1;			# Index of the last entry in @co
@@ -444,7 +481,7 @@ sub mkparaarray($@) {
 	foreach $chunk ( @chunks ) {
 	    my $type = substr($chunk,0,2);
 	    my $text = substr($chunk,2);
-	    
+
 	    if ( $type eq 'sp' ) {
 		push(@para, [$in_e?1:0, ' ']);
 	    } elsif ( $type eq 'da' ) {
@@ -558,7 +595,7 @@ for ( $i = 0 ; $i < $npara ; $i++ ) {
 	push(@tocptypes, 'toc0'.' :'.$sech.':'.$chap.' '.$secn.':');
 
 	unshift(@{$paras[$i]},
- 		[-5, $xref], [0,$chap.' '.$secn.':'], [0, ' ']);
+		[-5, $xref], [0,$chap.' '.$secn.':'], [0, ' ']);
     } elsif ( $ptype eq 'head' || $ptype eq 'subh' ) {
 	unless ( $xtype =~ /^\S+ (\S+) :(.*)$/ ) {
 	    die "Bad para";
@@ -591,11 +628,16 @@ unshift(@ptypes, @tocptypes); undef @tocptypes;
 #
 # Add copyright notice to the beginning
 #
-unshift(@paras,
-	[[0, $charcode{'copyright'}], [0, ' '], [0,$metadata{'year'}],
-	 [0, ' '], string2array($metadata{'author'})],
-	[string2array($metadata{'license'})]);
-unshift(@ptypes, 'norm', 'norm');
+@copyright_page =
+([[0, $charcode{'copyright'}],
+  [0, ' '], [0, $metadata{'year'}],
+  [0, ' '], string2array($metadata{'author'}),
+  [0, ' '], string2array($metadata{'copyright_tail'})],
+ [string2array($metadata{'license'})],
+ [string2array($metadata{'auxinfo'})]);
+
+unshift(@paras, @copyright_page);
+unshift(@ptypes, ('norm') x scalar(@copyright_page));
 
 $npara = scalar(@paras);
 
@@ -668,11 +710,11 @@ sub ps_break_lines($$) {
 	    my $refwidth = ps_width($refname, $BodyFont{fonts}->[0][1],
 				    \@NASMEncoding) *
 		($BodyFont{fonts}->[0][0]/1000);
-	    
+
 	    @ls = ps_flow_lines($linewidth-$ntoc*$psconf{tocind}-
 				$psconf{tocpnz}-$refwidth,
 				\%BodyFont, $ptype, @data);
-	    
+
 	    # Auxilliary data: for the first line, the cross reference symbol
 	    # and the reference name; for all lines but the first, the
 	    # reference width; and for the last line, the page number
@@ -723,7 +765,7 @@ $nlines = scalar(@pslines);
 #
 sub ps_break_pages($$) {
     my($startline, $endline) = @_;
-    
+
     # Paragraph types which should never be broken
     my $nobreakregexp = "^(chap|appn|head|subh|toc.|idx.)\$";
     # Paragraph types which are heading (meaning they should not be broken
@@ -756,22 +798,22 @@ sub ps_break_pages($$) {
 	    $columnstart = $curypos;
 	    $curcolumn = 0;
 	}
-    
+
 	# Adjust position by the appropriate leading
 	$curypos += $$linfo[3]->{leading};
-	
+
 	# Record the page and y-position
 	$$linfo[4] = $curpage;
-	$$linfo[5] = $curypos; 
+	$$linfo[5] = $curypos;
 	$$linfo[6] = $curcolumn if ( defined($columnstart) );
-	
+
 	if ( $curypos > $upageheight ) {
 	    # We need to break the page before this line.
 	    my $broken = 0;		# No place found yet
 	    while ( !$broken && $pslines[$i]->[0]->[4] == $curpage ) {
 		my $linfo = $pslines[$i]->[0];
 		my $pinfo = $pslines[$i-1]->[0];
-		
+
 		if ( $$linfo[1] == 2 ) {
 		    # This would be an orphan, don't break.
 		} elsif ( $$linfo[1] & 1 ) {
@@ -1128,7 +1170,7 @@ $curpage = 2;
 ps_start_page();
 foreach $line ( @pslines ) {
     my $linfo = $line->[0];
-    
+
     if ( $$linfo[4] != $curpage ) {
         ps_end_page($curpage > 2);
         ps_start_page();
