@@ -1,6 +1,6 @@
 /* ----------------------------------------------------------------------- *
  *   
- *   Copyright 1996-2009 The NASM Authors - All Rights Reserved
+ *   Copyright 1996-2011 The NASM Authors - All Rights Reserved
  *   See the file AUTHORS included with the NASM distribution for
  *   the specific copyright holders.
  *
@@ -754,6 +754,15 @@ struct ofmt {
     void (*cleanup) (int debuginfo);
 };
 
+/*
+ * Output format driver alias
+ */
+struct ofmt_alias {
+    const char  *shortname;
+    const char  *fullname;
+    struct ofmt *ofmt;
+};
+
 extern struct ofmt *ofmt;
 extern FILE *ofile;
 
