@@ -1,6 +1,6 @@
 /* ----------------------------------------------------------------------- *
  *   
- *   Copyright 1996-2009 The NASM Authors - All Rights Reserved
+ *   Copyright 1996-2010 The NASM Authors - All Rights Reserved
  *   See the file AUTHORS included with the NASM distribution for
  *   the specific copyright holders.
  *
@@ -48,7 +48,7 @@
 #include "insns.h"
 
 int globalbits = 0;    /* defined in nasm.h, works better here for ASM+DISASM */
-efunc nasm_malloc_error;	/* Exported for the benefit of vsnprintf.c */
+static vefunc nasm_verror;	/* Global error handling function */
 
 #ifdef LOGALLOC
 static FILE *logfp;
@@ -72,12 +72,30 @@ void tolower_init(void)
 	nasm_tolower_tab[i] = tolower(i);
 }
 
-void nasm_set_malloc_error(efunc error)
+void nasm_set_verror(vefunc ve)
+{
+    nasm_verror = ve;
+}
+
+void nasm_error(int severity, const char *fmt, ...)
+{
+    va_list ap;
+
+    va_start(ap, fmt);
+    nasm_verror(severity, fmt, ap);
+    va_end(ap);
+}
+
+void nasm_init_malloc_error(void)
 {
-    nasm_malloc_error = error;
 #ifdef LOGALLOC
     logfp = fopen("malloc.log", "w");
-    setvbuf(logfp, NULL, _IOLBF, BUFSIZ);
+    if (logfp) {
+        setvbuf(logfp, NULL, _IOLBF, BUFSIZ);
+    } else {
+        nasm_error(ERR_NONFATAL | ERR_NOFILE, "Unable to open %s", logfp);
+        logfp = stderr;
+    }
     fprintf(logfp, "null pointer is %p\n", NULL);
 #endif
 }
@@ -90,7 +108,7 @@ void *nasm_malloc(size_t size)
 {
     void *p = malloc(size);
     if (!p)
-        nasm_malloc_error(ERR_FATAL | ERR_NOFILE, "out of memory");
+        nasm_error(ERR_FATAL | ERR_NOFILE, "out of memory");
 #ifdef LOGALLOC
     else
         fprintf(logfp, "%s %d malloc(%ld) returns %p\n",
@@ -107,7 +125,7 @@ void *nasm_zalloc(size_t size)
 {
     void *p = calloc(size, 1);
     if (!p)
-        nasm_malloc_error(ERR_FATAL | ERR_NOFILE, "out of memory");
+        nasm_error(ERR_FATAL | ERR_NOFILE, "out of memory");
 #ifdef LOGALLOC
     else
         fprintf(logfp, "%s %d calloc(%ld, 1) returns %p\n",
@@ -124,7 +142,7 @@ void *nasm_realloc(void *q, size_t size)
 {
     void *p = q ? realloc(q, size) : malloc(size);
     if (!p)
-        nasm_malloc_error(ERR_FATAL | ERR_NOFILE, "out of memory");
+        nasm_error(ERR_FATAL | ERR_NOFILE, "out of memory");
 #ifdef LOGALLOC
     else if (q)
         fprintf(logfp, "%s %d realloc(%p,%ld) returns %p\n",
@@ -161,7 +179,7 @@ char *nasm_strdup(const char *s)
 
     p = malloc(size);
     if (!p)
-        nasm_malloc_error(ERR_FATAL | ERR_NOFILE, "out of memory");
+        nasm_error(ERR_FATAL | ERR_NOFILE, "out of memory");
 #ifdef LOGALLOC
     else
         fprintf(logfp, "%s %d strdup(%ld) returns %p\n",
@@ -182,7 +200,7 @@ char *nasm_strndup(const char *s, size_t len)
 
     p = malloc(size);
     if (!p)
-        nasm_malloc_error(ERR_FATAL | ERR_NOFILE, "out of memory");
+        nasm_error(ERR_FATAL | ERR_NOFILE, "out of memory");
 #ifdef LOGALLOC
     else
         fprintf(logfp, "%s %d strndup(%ld) returns %p\n",
@@ -195,8 +213,7 @@ char *nasm_strndup(const char *s, size_t len)
 
 no_return nasm_assert_failed(const char *file, int line, const char *msg)
 {
-    nasm_malloc_error(ERR_FATAL, "assertion %s failed at %s:%d",
-		      msg, file, line);
+    nasm_error(ERR_FATAL, "assertion %s failed at %s:%d", msg, file, line);
     exit(1);
 }
 
@@ -369,7 +386,7 @@ int64_t readnum(char *str, bool *error)
      * cheat: since we know that all radices we use are even, we
      * can divide 2**63 by radix/2 instead.
      */
-    checklimit = 0x8000000000000000ULL / (radix >> 1);
+    checklimit = UINT64_C(0x8000000000000000) / (radix >> 1);
 
     /*
      * Calculate the highest allowable value for the last digit of a
@@ -396,9 +413,9 @@ int64_t readnum(char *str, bool *error)
     }
 
     if (warn)
-        nasm_malloc_error(ERR_WARNING | ERR_PASS1 | ERR_WARN_NOV,
-                          "numeric constant %s does not fit in 64 bits",
-                          str);
+        nasm_error(ERR_WARNING | ERR_PASS1 | ERR_WARN_NOV,
+		   "numeric constant %s does not fit in 64 bits",
+		   str);
 
     return result * sign;
 }
@@ -413,7 +430,7 @@ int64_t readstrnum(char *str, int length, bool *warn)
     str += length;
     if (globalbits == 64) {
         for (i = 0; i < length; i++) {
-            if (charconst & 0xFF00000000000000ULL)
+            if (charconst & UINT64_C(0xFF00000000000000))
                 *warn = true;
             charconst = (charconst << 8) + (uint8_t)*--str;
         }
@@ -513,8 +530,7 @@ size_t fwritezero(size_t bytes, FILE *fp)
     return count;
 }
 
-void standard_extension(char *inname, char *outname, char *extension,
-                        efunc error)
+void standard_extension(char *inname, char *outname, char *extension)
 {
     char *p, *q;
 
@@ -531,13 +547,13 @@ void standard_extension(char *inname, char *outname, char *extension,
             p++;                /* go back to end if none found */
     if (!strcmp(p, extension)) {        /* is the extension already there? */
         if (*extension)
-            error(ERR_WARNING | ERR_NOFILE,
-                  "file name already ends in `%s': "
-                  "output will be in `nasm.out'", extension);
+            nasm_error(ERR_WARNING | ERR_NOFILE,
+		       "file name already ends in `%s': "
+		       "output will be in `nasm.out'", extension);
         else
-            error(ERR_WARNING | ERR_NOFILE,
-                  "file name already has no extension: "
-                  "output will be in `nasm.out'");
+            nasm_error(ERR_WARNING | ERR_NOFILE,
+		       "file name already has no extension: "
+		       "output will be in `nasm.out'");
         strcpy(outname, "nasm.out");
     } else
         strcpy(p, extension);
@@ -554,7 +570,7 @@ static const char *prefix_names[] = {
 const char *prefix_name(int token)
 {
     unsigned int prefix = token-PREFIX_ENUM_START;
-    if (prefix > elements(prefix_names))
+    if (prefix >= ARRAY_SIZE(prefix_names))
 	return NULL;
 
     return prefix_names[prefix];
@@ -642,3 +658,152 @@ char *nasm_strcat(const char *one, const char *two)
     strcpy(rslt + l1, two);
     return rslt;
 }
+
+/* skip leading spaces */
+char *nasm_skip_spaces(const char *p)
+{
+    if (p)
+        while (*p && nasm_isspace(*p))
+            p++;
+    return (char *)p;
+}
+
+/* skip leading non-spaces */
+char *nasm_skip_word(const char *p)
+{
+    if (p)
+        while (*p && !nasm_isspace(*p))
+            p++;
+    return (char *)p;
+}
+
+/* zap leading spaces with zero */
+char *nasm_zap_spaces_fwd(char *p)
+{
+    if (p)
+        while (*p && nasm_isspace(*p))
+            *p++ = 0x0;
+    return p;
+}
+
+/* zap spaces with zero in reverse order */
+char *nasm_zap_spaces_rev(char *p)
+{
+    if (p)
+        while (*p && nasm_isspace(*p))
+            *p-- = 0x0;
+    return p;
+}
+
+/* zap leading and trailing spaces */
+char *nasm_trim_spaces(char *p)
+{
+    p = nasm_zap_spaces_fwd(p);
+    nasm_zap_spaces_fwd(nasm_skip_word(p));
+
+    return p;
+}
+
+/*
+ * return the word extracted from a stream
+ * or NULL if nothing left
+ */
+char *nasm_get_word(char *p, char **tail)
+{
+    char *word = nasm_skip_spaces(p);
+    char *next = nasm_skip_word(word);
+
+    if (word && *word) {
+        if (*next)
+            *next++ = '\0';
+    } else
+        word = next = NULL;
+
+    /* NOTE: the tail may start with spaces */
+    *tail = next;
+
+    return word;
+}
+
+/*
+ * Extract "opt=val" values from the stream and
+ * returns "opt"
+ *
+ * Exceptions:
+ * 1) If "=val" passed the NULL returned though
+ *    you may continue handling the tail via "next"
+ * 2) If "=" passed the NULL is returned and "val"
+ *    is set to NULL as well
+ */
+char *nasm_opt_val(char *p, char **val, char **next)
+{
+    char *q, *opt, *nxt;
+
+    opt = *val = *next = NULL;
+
+    p = nasm_get_word(p, &nxt);
+    if (!p)
+        return NULL;
+
+    q = strchr(p, '=');
+    if (q) {
+        if (q == p)
+            p = NULL;
+        *q++='\0';
+        if (*q) {
+            *val = q;
+        } else {
+            q = nasm_get_word(q + 1, &nxt);
+            if (q)
+                *val = q;
+        }
+    } else {
+        q = nasm_skip_spaces(nxt);
+        if (q && *q == '=') {
+            q = nasm_get_word(q + 1, &nxt);
+            if (q)
+                *val = q;
+        }
+    }
+
+    *next = nxt;
+    return p;
+}
+
+/*
+ * initialized data bytes length from opcode
+ */
+int idata_bytes(int opcode)
+{
+    int ret;
+    switch (opcode) {
+    case I_DB:
+        ret = 1;
+        break;
+    case I_DW:
+        ret = 2;
+        break;
+    case I_DD:
+        ret = 4;
+        break;
+    case I_DQ:
+        ret = 8;
+        break;
+    case I_DT:
+        ret = 10;
+        break;
+    case I_DO:
+        ret = 16;
+        break;
+    case I_DY:
+        ret = 32;
+        break;
+    case I_none:
+        ret = -1;
+        break;
+    default:
+        ret = 0;
+        break;
+    }
+    return ret;
+}
